
**************************************************************************;
* Project: PRISM, Concomitant Vaccines           						  ;
* Program: tree_extraction_SCRIsas                       				  ;
* Version:V1.2.0                                                          ;
* Date : 05/4/2016                                                        ;
* Input data: INDATA: INDATA.&demtable                                    ;
*                    INDATA.&enrtable                                     ;
*                    INDATA.&diatable                                     ;
*                    INDATA.&PROCTABLE                                    ;
*                    INDATA.&distable                                     ; 
*  			  Files provided with the program:                            ;
*                    site_info                                            ;
*                    general_input                                        ;
*                    subgroup                                             ;
*                    dxtree_input                                         ;
*                    dxtree_priority                                      ;  
*                    concocmitant_group                                   ;
*                    group_exclusion                                      ;
*                    vaccine_group_enc_setting                            ;  
*                    id10_id9_map                                         ;
*                    Macros: create_episodes_spans_2.sas    			  ; 
*                            isdata.sas                                   ;
*                            ms_delpatients.sas                           ; 
*                            ms_time_signature.sas                        ;
*                                                                         ;
* Output data to be sent along with *log file:                            ;
*                 MSOC.self_control_orig_dx                               ;
*                 MSOC.elig_vacc_doses                                    ;
*                 MSOC.vacc_wkday                                         ;
*                 MSOC.vacc_age                                           ;  
*                 MSOC.dx_wkday                                           ;  
*                 msoc.orig_dx_wkday                                      ;
*                 msoc.ms_time_signature                                  ;
**************************************************************************;


	 

 %ms_starttimer(START=MAINSTART);

*** Site specific input information;

proc sql noprint;
  select cohort_start_dt FORMAT=8. , cohort_end_dt FORMAT=8. 
            , STRIP(PUT(ASO_EXCL_FLAG,1.)) into :cohort_start_dt,:cohort_end_dt,: aso_excl_flag
    from &site_info.
	  WHERE UPCASE(DPID)=UPCASE("&DPID.")
	  		AND   UPCASE(siteid)=UPCASE("&siteid.")
		;
 TITLE "Cohort Dates" ;
quit; 



*** To check requester & site's inputs regarding exclusion ASO population;

*** If needed  - could be used for any exclusion list - just to be clear the name of the flag and file probably should be changed;

%PUT aso_excl_flag=&aso_excl_flag.;

 %ISDATA(dataset=&ASO_EXCL_LIST.);

 %MACRO chk_aso_excl;

 %IF  &aso_excl_flag.=1 AND  &NOBS.=0  %THEN
              %DO;
	  %PUT WARNING: The  aso_excl_flag=1 but the dataset ASO_EXCL_LIST does not exist or has no records ! ;
			   %END;                 

 %MEND;

 %chk_aso_excl;


 * Process input tables;

 * Set defaults and do some defensive coding for general input variables;

 
 DATA GENERAL_INPUT;
    SET &GENERAL_INPUT.;
	
	IF	 Medcov=" " THEN Medcov="Y";	
	 Medcov=UPCASE(Medcov) ;
	 Drugcov=UPCASE(Drugcov) ;

	  Age_begin=COALESCE(Age_begin,0);
	  Age_finish=COALESCE(Age_finish,160);
	  AGE_PERIOD=COALESCEC(AGE_PERIOD,"YEAR");
	  Incident_level=COALESCE(Incident_level,3);

	  Enr_gap=COALESCE(Enr_gap,45);
	  Enr_ramp_up=COALESCE(Enr_ramp_up,183);
	  AE_wash_up=COALESCE(AE_wash_up,183);
	  F_up_window_st=COALESCE(F_up_window_st,1);
	  F_up_window_end=COALESCE(F_up_window_end,56);

	  birth_enr_dist_allowed=COALESCE(birth_enr_dist_allowed,30);
	  enr_birth_grace_period=COALESCE(enr_birth_grace_period,45);
	  if AE_wash_up<F_up_window_end+1 then do;
	  PUT 'WARNING: The length of AE_wash_up should be >= F_up_window_end+1 !!!';  
	  end;
RUN;

data grp_inc_encounter_setting;
 set &vaccine_group_enc_setting;
   Incident_enc_setting=UPCASE(Incident_enc_setting);
    run;
	

*** Create unique combination of group & inclusion and exclusion subgroups;


proc sql;
  create table grp_subgrp as
    select distinct group, subgroup 
      from &CONCOMITANT_GROUP.;
	  create table excl_group as
	  select distinct vaccine_group, subgroup
	    from &group_exclusion.;
	  quit;


	  
	  *** Just to be sure there is a right dx_mlccs structured file;
	 
proc sql;
  create table dx_mlccs as
  select distinct dx, dx_codetype, mlccs1, mlccs2, mlccs3, mlccs4, mlccs5
    from &dxtree_input.
	  order by dx, dx_codetype;
quit;

	  proc transpose data=dx_mlccs out=tr_dx_mlccs;
	    by dx dx_codetype;
		 var mlccs1-mlccs5;
		 quit;

     data dx_mlccs_long(drop=_name_);
	   set tr_dx_mlccs(rename=(col1=mlccs_incident));
	    level=INPUT(put(compress(_name_,' mlccs'),$4.),4.);
  run;



  *** Attach priority to all tree diagnosis of interest for each vaccine group according to list of this group parameters;
 
  PROC SQL ;
    CREATE TABLE  dx_mlccs_for_inc_level_by_grp AS
	  SELECT   A.vaccine_group
                     , A.Incident_level
					  ,B.mlccs_incident
	                  ,A.DX
                      ,A.DX_CODETYPE
					  ,A.priority
                      
	      FROM 	          &dxtree_priority.           A
		  INNER JOIN  dx_mlccs_long	                        B
		                   ON A.DX=B.DX
						AND  A.DX_CODETYPE=B.DX_CODETYPE
						AND A.Incident_level=B.level
				ORDER BY A.vaccine_group
                                   ,B.mlccs_incident
								   ,A.priority
								   ,B.DX
					;
	QUIT;

data id9_dxtree_input;
  set &dxtree_input.(DROP=mlccs: );
  orig_dx=dx;
  orig_dx_codetype=dx_codetype;
run;

data dxtree_input;
  set id9_dxtree_input &id10_id9_map.;
  run;


PROC SORT DATA=dxtree_input
                       OUT=dx_tree_input_codes_srt NODUPKEY;
 BY orig_dx orig_dx_codetype;
RUN;

  *** Check what code categories reflect exposure codes and extract the records 
     containing these exposure codes and dxtree codes within defined cohort time window from correspondent files;


 %macro exp;
  %LOCAL exp_dx_events_tbl_name exp_px_events_tbl_name exp_rx_events_tbl_name;

data dx_exp_codes 
		px_exp_codes 
        rx_exp_codes(rename=(code=NDC))
    ;

  set &subgroup.;
	  if         code_category='DX' then output dx_exp_codes;
	  else if code_category='PX' then output px_exp_codes;
	  else if code_category='RX' then output rx_exp_codes;
run;
   
proc sql NOPRINT;
  select STRIP(PUT(count(*),12.)) as nobs into:dx_codes_cnt
    from dx_exp_codes;
 select STRIP(PUT(count(*),12.)) as nobs into:px_codes_cnt
    from px_exp_codes;
 select STRIP(PUT(count(*),12.)) as nobs into:rx_codes_cnt
    from rx_exp_codes;
 quit;

/* *** To put number of exposure codes of each category into log;*/

 %PUT dx_codes_cnt=&dx_codes_cnt.;  
 %PUT px_codes_cnt=&px_codes_cnt.; 
 %PUT rx_codes_cnt=&rx_codes_cnt.; 

/*** if dx codes are listed as exposure  - extract correspondent data from INDATA diagnosis file;*/
/*** Otherwise extract the data for tree diagnosis only;*/



%if &dx_codes_cnt ne 0 %then %do;


PROC SORT DATA=dx_exp_codes(rename=(code=orig_dx code_type=orig_dx_codetype) drop=code_category) 
						OUT=dx_exp_codes_srt ;
 BY orig_dx orig_dx_codetype;
RUN;

/*** Combine exposure and tree diags by merge to avoid duplicates during extraction process;*/
data dx_all_input_codes;
  MERGE dx_tree_input_codes_srt(IN=intree) 
                dx_exp_codes_srt(IN=inexp)
      ;
    BY orig_dx orig_dx_codetype;
	 in_tree=intree;
	 in_exp=inexp;

	 IF subgroup=. THEN subgroup=-1;	  
  run;
    %LET exp_dx_events_tbl_name=exp_dx_events;
%END;

%else %do;
data dx_all_input_codes;
  set dx_tree_input_codes_srt;
   in_tree=1;
   in_exp=0;
   subgroup=-1;	
  run; 
     %LET exp_dx_events_tbl_name=%STR();
  %end;

  PROC FREQ DATA=dx_all_input_codes;
   TABLES in_tree*in_exp*subgroup/LIST NOCUM;
    TITLE "Sources of DX codes";
   RUN; 

   
 /*** To create the list of unique diagnosis specific to the data partner from INDATA diag file ;*/

proc sql;
   create table unique_mscdm_dx_codes as
   select distinct DX, dx_codetype
     from INDATA.&diatable
;
 quit;
  		   			
	
/*** From all unique DP's diagnosis select only those presented in tree or exposure list per request;*/
	
proc sql;
  create table dx_input_codes_in_mscdm_form as
  select a.dx
            ,a.dx_codetype,b.subgroup
			,b.in_tree,b.in_exp
   from unique_mscdm_dx_codes as    a
		  , dx_all_input_codes                 as b
     where compress(upcase(a.dx),'. ')=compress(upcase(b.orig_dx),'. ') and a.dx_codetype=b.orig_dx_codetype;
	 quit;

	 	
/*** Extract all events with defined list of diagnosis of interest;*/
	
proc sql;
  create table all_dx_events as
  select  a.patid, a.adate,a.dx, a.dx_codetype, a.enctype
		    , b.subgroup
			,b.in_tree,b.in_exp
    from INDATA.&diatable                                               as a
		  , dx_input_codes_in_mscdm_form                        as b 
	  where a.dx=b.dx and a.dx_codetype=b.dx_codetype
		 AND &cohort_start_dt.<=adate<=&cohort_end_dt.
		;
quit;

/*** Split extracted events into separate files - tree and exposure;*/
	
	   data exp_dx_events(keep=patid adate dx dx_codetype subgroup) 
				tree_dx_events(keep= patid adate enctype dx  dx_codetype rename=(dx=orig_dx dx_codetype=orig_dx_codetype));
	     set all_dx_events;

		 if in_exp then output exp_dx_events;
		  if in_tree then output tree_dx_events;          
		run;
 



/*** if px codes are listed as exposure  - extract data from INDATA procedure file;*/
/*** Find events of interest similar to dx exposure events above; */

%if &px_codes_cnt ne 0 %then %do;

 proc sql;
   create table unique_mscdm_px_codes as
   select distinct px
     from INDATA.&PROCTABLE
   ;
 quit;

proc sql;
  create table px_input_codes as
  select DISTINCT a.*
                            ,b.code_type  as px_codetype
                            ,b.subgroup
   from unique_mscdm_px_codes as a
		  ,px_exp_codes                as b
     where compress(upcase(a.px),'. ')=compress(upcase(b.code),'. ');
quit;


proc sql;
  create table exp_px_events as
  select b.subgroup, a.patid, a.adate,a.px, a.px_codetype
    from INDATA.&PROCTABLE as a
          , px_input_codes              as b
	  where   a.px=b.px and a.px_codetype=b.px_codetype
		  AND &cohort_start_dt.<=adate<=&cohort_end_dt.
			;
	   quit;



	     %LET exp_px_events_tbl_name=exp_px_events;
%end;
%else  %LET exp_px_events_tbl_name=%STR();

	 /*** if rx codes are listed as exposure  - extract data from INDATA dispensing file;*/
/*** Find events of interest similar to dx & px exposure events above; */

%if &rx_codes_cnt ne 0 %then %do;

proc sql;
  create table exp_rx_events(rename=(rxdate=adate)) as
  select b.subgroup, a.patid, a.rxdate, a.NDC
    from INDATA.&distable as a
            ,rx_exp_codes as b                                                          
	  where a.NDC=b.NDC 
	  AND &cohort_start_dt.<=rxdate<=&cohort_end_dt.
   ;
 quit;

   	   %LET exp_rx_events_tbl_name=exp_rx_events;
%end;
%else  %LET exp_rx_events_tbl_name=%STR();

/*** Create exposure events structure file with 0 observation, but containing columns of interest;*/

DATA all_exp_events_structure;
   SET INDATA.&diatable(OBS=0 KEEP=patid adate dx dx_codetype )
	        INDATA.&PROCTABLE(OBS=0 KEEP=px px_codetype )
            INDATA.&distable(OBS=0 KEEP=NDC)
			grp_subgrp(OBS=0 KEEP=subgroup)
	;
 RUN;


 /*** Combine all exposure events;*/

DATA all_exp_events;
  SET all_exp_events_structure
          &exp_dx_events_tbl_name.
          &exp_px_events_tbl_name.
          &exp_rx_events_tbl_name.
    ;
RUN;

/*** Exclude ASO patients if (properly) requested (it means if flag=1 then the file should exist);*/

%IF  &aso_excl_flag.=1 %THEN 
                                  %DO;
%ms_delpatients(datafile=all_exp_events, ptsfile=&ASO_EXCL_LIST.,    
                               outfile=all_exp_events);
							      %END;

%mend exp;

%exp;


      *  Create table all_exp_events_by_vacc_grp which has all parameters for specified
         concomitant group attached to each exposure record based on subgroups of inclusion;
       * It will be used in several places below;

PROC SQL;
   	 CREATE TABLE all_exp_events_by_vacc_grp	AS 
	    SELECT DISTINCT  B.vaccine_group,
		b.group,
		b.primary,
		b.vaccine_order
		
                                         ,A.PatID
		                                ,A.ADATE
				FROM              all_exp_events		    A
				INNER JOIN     &concomitant_group.       		  B
	                  ON A.subgroup=B.subgroup
				  ORDER BY 	   B.vaccine_group
                                         ,A.PatID
		                                , A.ADATE
				 ;
QUIT;


      *  Create table exp_events_excl which has vaccine_group attached to exposure records based on exclusion subgroups; 
proc sql;
create table exp_events_excl as
  select distinct  B.vaccine_group
		          ,A.PatID
		          ,A.ADATE
				 
				FROM              all_exp_events		    A
				INNER JOIN     &group_exclusion       		  B
	                  ON A.subgroup=B.subgroup
				  ORDER BY 	   B.vaccine_group
                                         ,A.PatID
		                                , A.ADATE;
quit;


*** Limit exposure records for each vaccine_group to primary vaccine only;

PROC SQL;
   	 CREATE TABLE exp_pat_group_prim	AS 
	    SELECT DISTINCT  vaccine_group
                                         ,PatID, adate
				FROM  all_exp_events_by_vacc_grp
				  where primary=1;
                
				 ;
QUIT;
 



*** Create patient-group level table combining primary exposure data and general input info;

PROC SQL;
 CREATE TABLE   exp_pat_group_parms AS
   SELECT  
        A.PatID
	   ,A.adate
       ,B.*
	    FROM 		   exp_pat_group_prim             A
		INNER JOIN general_input           B
		      ON    A.vaccine_group=B.vaccine_group
		ORDER BY  B.vaccine_group, A.PatID, adate
				;
QUIT;

*** Select incident exposures only for each vaccine group according to specified exp_wash_up;
 
data exp_pat_group_parms2;
 set exp_pat_group_parms;
  by vaccine_group PatID adate;
    retain check_dt;
	if first.patid then do;
                        check_dt=adate;
                        dist=.;
						end;
	else do;
         dist=adate-check_dt;
		 check_dt=adate;
		 end;
		 
		  run;

		  data exp_pat_group_parms3;
		    set exp_pat_group_parms2;
              by vaccine_group PatID adate;
                 if ~first.patid and dist<=exp_wash_up then delete;
run; 

 
*** Add enrollment and demo info;


proc sql;
  create table exp_enroll_demo as
  select  C.*
            ,a.ENR_START
			,a.ENR_END
            ,b.birth_date 
    from  		INDATA.&enrtable as a
    INNER JOIN   exp_pat_group_parms3 as C
	   ON      a.PatID=C.PatID
	  AND (A.MEDCOV=C.MEDCOV)
	  AND (A.DRUGCOV=C.DRUGCOV OR C.DRUGCOV IS NULL)
   INNER JOIN  INDATA.&demtable as b
	  ON C.patid=b.patid 
	  and b.birth_date is not Null
   ;
   QUIT;
  


*** Roll up enrollment records with defined memgap;

%create_episodes_spans_2(tbl_name_in=exp_enroll_demo
                         ,tbl_name_out=roll_up_enr(RENAME=(enr_start_span=enr_start_dt  enr_end_span=enr_end_dt)) 
                          ,by_vars=vaccine_group patid
                          ,eff_dt_var_nm=ENR_start
                          ,exp_dt_var_nm=enr_end
                          ,grace_period_var_nm=enr_gap);



*** Select only records that meet enrollment/demo criteria;


proc sort data=roll_up_enr;
   by vaccine_group patid;
run;
 

data enroll_demo
           excl;
   set roll_up_enr;
  *set exp_pat_group_parms_age;  
    LENGTH excl_reason $ 32;
	 
	FORMAT age_begin_dt  age_finish_dt mmddyy10.;

	
	   orig_enr_start_dt=enr_start_dt ;
	   flag_enr_from_birth=0;

  if enr_start_dt-birth_date<-birth_enr_dist_allowed then do;
                                                excl_reason='enr_before_dob';
                                                output excl;
												end;
											

	 ELSE IF 	enr_start_dt-birth_date<=enr_birth_grace_period	  then 
         DO;
      enr_start_dt=birth_date;         *Reset enr_start_dt to birth_date;   *** new;
	  flag_enr_from_birth=1;				 ;
	    END;
                    
  ELSE; * do nothing;
		
            age_begin_dt=intnx(age_period, birth_date, age_begin, 'SAMEDAY');
			age_finish_dt=intnx(age_period, birth_date, age_finish+1, 'SAMEDAY')-1;

			if age_begin_dt>enr_end_dt or age_finish_dt<enr_start_dt then do;
			                                              excl_reason='enr_age       ';
														  output excl;
														   end;
  
 if enr_end_dt-enr_start_dt>=max(Exp_wash_up,(AE_wash_up-F_up_window_st))+f_up_window_end +1 then flag_ramp=1;


if (flag_ramp or flag_enr_from_birth)=1 AND excl_reason=" "  then  output enroll_demo;
									else do;                 
                                      excl_reason='insufficient_enr_length    '; 
								      output excl;
									end; 
RUN;

   
*** Define contributed time periods (when vaccine dates meet enrollment criteria);

data enroll_demo2 
            excl2;
  set enroll_demo;
   by vaccine_group patid enr_start_dt;

  FORMAT enr_contrib_st  enr_contrib_end  mmddyy10.;

  if flag_enr_from_birth=1 then enr_contrib_st=max(&cohort_start_dt., age_begin_dt);
	  else enr_contrib_st=max(&cohort_start_dt.+max(Exp_wash_up,(AE_wash_up-F_up_window_st)), enr_start_dt+max(Exp_wash_up,(AE_wash_up-F_up_window_st)), age_begin_dt);

	  enr_contrib_end=min(&cohort_end_dt., enr_end_dt, age_finish_dt +f_up_window_end);

format enr_contrib_st enr_contrib_end mmddyy10.;

contrib_wind_length=enr_contrib_end-enr_contrib_st;

if enr_contrib_end-enr_contrib_st<f_up_window_end then do;
                                                    excl_reason='insufficient_length_contrib_win';
                                                     output excl2;
													end;

else output enroll_demo2;

run;

proc sort data=enroll_demo2;
  by vaccine_group patid enr_contrib_st enr_contrib_end;
 run;



	***  Select only exposures for patient of appropriate age with eligible enrollment within cohort tw;
*var patid &cohort_start_dt enr_start_dt age_begin_dt enr_contrib_end f_up_window_end;
* some variables added for validation purposes only;

 proc sql;
  create table vacc as
             select  a.PatID,a.ADATE
                       ,b.vaccine_group, b.birth_date, b.enr_start_dt
                       ,b.enr_contrib_st, b.enr_contrib_end, b.flag_enr_from_birth, b.flag_ramp
                      ,b.age_period, b.AE_wash_up, b.f_up_window_st, b.f_up_window_end, b.exp_wash_up
					  , b.enr_end_dt, b.age_begin_dt, b.age_finish_dt, b.enr_ramp_up

    from 			 exp_pat_group_parms3 as a
    INNER JOIN enroll_demo2   as b
	                 ON a.patid=b.patid
					AND a.vaccine_group=b.vaccine_group
					AND enr_contrib_st<=a.adate<=b.enr_contrib_end-b.f_up_window_end

           order by b.vaccine_group, a.patid, a.adate
              ;
quit;



*** Create set of flags to check criteria;

data vacc2(rename=(adate=vacc_dt)) ;
  set vacc;
    by vaccine_group patid adate;
	vacc_wkday=weekday(adate);

	  age_at_vacc_dt=intck(age_period,birth_date, adate, 'c');
      	 

	 flag_right_age_at_vacc_dt=(age_begin_dt<=adate<=age_finish_dt);
	 flag_enr_at_vacc_dt=(enr_start_dt<=adate<=enr_end_dt);
	 flag_enr_ramp_at_vacc_dt=(flag_enr_at_vacc_dt AND adate-enr_start_dt>=max(EXP_wash_up,(AE_wash_up-F_up_window_st)));
	 flag_enr_at_f_up_window_st=(enr_start_dt<=adate+f_up_window_st<=enr_end_dt);
	 flag_enr_at_f_up_window_end=(enr_start_dt<=adate+f_up_window_end<=enr_end_dt);
	 
	
run;
 

				 TITLE1  "Combination of flag_enr_from_birth*flag_enr_ramp_at_vacc_dt should have 1 for at least one of them";
				  TITLE2 "All other flags should be 1" ;

PROC FREQ DATA=vacc2; 

  TABLES  flag_enr_from_birth*flag_enr_ramp_at_vacc_dt
				 flag_right_age_at_vacc_dt  flag_enr_at_vacc_dt 
                  flag_enr_at_f_up_window_st  flag_enr_at_f_up_window_end
                  /MISSING LIST;
RUN;

*** Exclude records with prior vaccination based on exclusion criteria;

proc sql;
  create table vacc_grp_excl as
  select distinct a.*, b.adate as excl_adate
    from vacc2 as a left join exp_events_excl as b
        on a.vaccine_group=b.vaccine_group and a.patid=b.patid and a.vacc_dt>b.adate;
		create table vacc3 as
		select *
		  from vacc_grp_excl
		    where excl_adate=.;
			quit;



TITLE1;
TITLE2;

*** Add non_primary exposure to the selected primary ones for each vaccine_group;

proc sql;
     create table vaccine_dt_all as
	 select distinct a.*, b.age_period, b.birth_date, 1 as flag, b.enr_start_dt, b.enr_end_dt
	  from all_exp_events_by_vacc_grp as a, vacc3 as b
	   where a.vaccine_group=b.vaccine_group and a.patid=b.patid and a.adate=b.vacc_dt
 order by patid, adate, vaccine_group ;
quit;


proc sql;
  create table combo_template as
  select distinct a.patid, a.adate, a.enr_start_dt, a.enr_end_dt, b.vaccine_group, b.vaccine_order, a.age_period, 
                  a.birth_date,0 as flag
    from vaccine_dt_all as a, &concomitant_group. as b
 order by patid , adate, vaccine_group;
	quit;

proc sql;
     create table vaccine_dt_all_2 as
	 select distinct a.*,  sum(a.flag, b.flag) as combo_flag
	  from combo_template as a left join vaccine_dt_all as b
	    on a.vaccine_group=b.vaccine_group & a.vaccine_order=b.vaccine_order & a.patid=b.patid and a.adate=b.adate 
 order by patid, adate, vaccine_group, vaccine_order, birth_date, age_period ;
quit;

 proc transpose data=vaccine_dt_all_2(drop=vaccine_order) out=vaccine_dt_all_3(drop= _name_) prefix=V;

by patid adate enr_start_dt enr_end_dt vaccine_group birth_date  age_period ;
   var combo_flag;
    run;

 data vaccine_dt_all_4;
  set vaccine_dt_all_3;
  where v1=1;
	vacc_comb=compress(v1||v2||v3);

		vacc_wkday=weekday(adate);
	  age_at_vacc_dt=intck(age_period,birth_date, adate, 'c');
	 run;

*** Define number of eligible vaccine doses for each group of interest;
*** This file should be sent to MSOC;

proc sql;
 create table MSOC.elig_vacc_doses as
  select distinct a.vaccine_group, b.vaccine_group_name, vacc_comb, count(*) as elig_vacc_dose_cnt
    from vaccine_dt_all_4 as a, general_input as b
	 where a.vaccine_group=b.vaccine_group
      group by a.vaccine_group, vacc_comb;
quit;

*** Define the frequency of the day of the week and patient age at the day of vaccination;
*** Result files should be sent to MSOC;
proc sort data=vaccine_dt_all_4;
 by vaccine_group vacc_comb;
  run;

proc freq data=vaccine_dt_all_4 noprint;
  by vaccine_group vacc_comb;
  tables vacc_wkday/missing out=msoc.vacc_wkday;
 tables age_period*age_at_vacc_dt/missing out=msoc.vacc_age;
 run;


 *** Define the time period of diagnosis consideration;

 data enroll_demo4;
   set vacc3;
     
     AE_win_end_dt=vacc_dt+f_up_window_end;

   * Calculate  outcome of interest start date. ; 
   outcome_win_start_dt=vacc_dt+f_up_window_st;

 age_at_outc_win_start_days=outcome_win_start_dt-birth_date; **** new;

 
if flag_enr_from_birth=1 & age_at_outc_win_start_days<AE_wash_up then AE_win_start_dt=birth_date;	   **** new;
 ELSE AE_win_start_dt=outcome_win_start_dt-AE_wash_up;                                              **** new;


   * The outcome end date is the same as AE_win_end_dt;
   
  format birth_date AE_win_start_dt AE_win_end_dt outcome_win_start_dt mmddyy10.;
  run;


				   *Process outcomes (dx tree events); 


proc sql ;
  create table dx_tree_events_for_exp_pat as
  select DISTINCT A.PATID, A.ADATE, A.orig_DX, A.orig_DX_CODETYPE, A.ENCTYPE, b.dx, b.dx_codetype
    from tree_dx_events	 a, dxtree_input as b
	  where a.patid in
	   (select patid
	    from enroll_demo4) and compress(upcase(a.orig_dx),'.')=compress(upcase(b.orig_dx),'.')
              and a.orig_dx_codetype=b.orig_dx_codetype
 order by dx, dx_codetype;
quit;
	

PROC SQL ;
  CREATE TABLE dx_tree_vacc_group_events as
  SELECT 		 b.vaccine_group
			,A.PATID
                        ,A.ADATE
						,A.orig_dx
						,A.orig_dx_codetype
                        ,b.mlccs_incident
                        ,b.incident_level   
			,b.priority
			,b.DX
                        ,b.DX_CODETYPE 
                        ,A.ENCTYPE

    FROM   dx_tree_events_for_exp_pat as a
          INNER JOIN    dx_mlccs_for_inc_level_by_grp as b
	                ON compress(upcase(a.dx),'. ')=compress(upcase(b.dx),'. ')
                   AND a.dx_codetype=b.dx_codetype  
      ORDER BY b.vaccine_group, a.patid, a.adate,b.mlccs_incident,b.priority, b.dx, b.dx_codetype, 
               b.incident_level;
quit;
 


proc sql;
  create table diag_demo as
  select distinct b.vaccine_group, a.patid, a.adate, a.enctype, a.mlccs_incident ,A.orig_dx
						,A.orig_dx_codetype, a.dx, a.dx_codetype,a.incident_level   
						,a.priority, b.enr_start_dt, b.vacc_dt, b.AE_win_start_dt, b.f_up_window_st, b.f_up_window_end,  b.AE_wash_up
						,b.outcome_win_start_dt
    from dx_tree_vacc_group_events as a
           , enroll_demo4     as b 
	  where a.vaccine_group=b.vaccine_group  
             & a.patid=b.patid 
            & b.AE_win_start_dt<=a.adate<=AE_win_end_dt
                 ;  
quit;
 

*** Select potential incident diagnosis;
   

proc sql;
create table diag_demo_pot_inc as
	  select a.*
	    from diag_demo(where=( adate>=outcome_win_start_dt)) as a
               ,grp_inc_encounter_setting as c
		  where a.vaccine_group=c.vaccine_group 
                  & (a.enctype=c.incident_enc_setting OR  c.incident_enc_setting IS NULL)
		 ORDER BY vaccine_group, patid,vacc_dt, adate, mlccs_incident, priority
      ;
quit;


*** Select the highest priority diagnosis;

		  data diag_demo3;
		   set diag_demo_pot_inc;
		     by vaccine_group patid vacc_dt adate mlccs_incident priority;
			  if first.mlccs_incident;
			  run;


*** To check all possible prior dx to potential incident diagnosis of the same incident level; 

proc sql;
  create table diag_demo4 as
  select distinct a.*
                       , b.adate as prior_dt format=mmddyy10.
                       , (a.adate-b.adate) as days_from_prev_AE
                       , b.enctype as b_type
    from     diag_demo3 as a 
   left join diag_demo   as b
	  on a.patid=b.patid 
    and a.mlccs_incident=b.mlccs_incident 
    and a.adate>b.adate
 order by vaccine_group, patid, vacc_dt, mlccs_incident, adate, days_from_prev_AE;
quit;

*** Select incident diagnosis if there were no prior one within requested AE wash_up_period
 on the same MLCCS level (incident_level) defined in the general input file;
*** Define the weekdays distribution of incident diagnosis;


data diag_demo5
		excluded_due_to_prev_AE
			;
  set diag_demo4;
  by vaccine_group patid vacc_dt mlccs_incident adate days_from_prev_AE;
	
	days_from_exp_to_outcome=(adate-vacc_dt);
   dx_wkday=weekday(adate);

flag_outc_in_f_up_window=(f_up_window_st <=days_from_exp_to_outcome <=f_up_window_end);

  IF  days_from_prev_AE >=0 THEN flag_prev_AE_inside_wash=(days_from_prev_AE <AE_wash_up);
  ELSE flag_prev_AE_inside_wash=.;

  flag_secondary_outc_in_a_day=(first.adate=0);
						

  if first.adate and (days_from_prev_AE=. or days_from_prev_AE>=AE_wash_up)  
		then output diag_demo5;
  else 		 output excluded_due_to_prev_AE;
run;

  TITLE1 "Flags for selected outcomes";
proc freq data=diag_demo5  ;
  tables flag_prev_AE_inside_wash  flag_outc_in_f_up_window/MISSING LIST;
 run;


  TITLE1 "Flags for excluded outcomes";
proc freq data=excluded_due_to_prev_AE  ;
  tables flag_prev_AE_inside_wash*flag_secondary_outc_in_a_day /MISSING LIST;
 run;
  
   TITLE1;

*** Attach vaccine concomitant information for each primary vaccination date;

  

proc sql;
  create table diag_demo6 as
  select a.*, b.vacc_comb
    from diag_demo5 as a, vaccine_dt_all_4 as b
	  where a.patid=b.patid & a.vacc_dt=b.adate & a.vaccine_group=b.vaccine_group
        order by vaccine_group, vacc_comb;
quit;
 

*** The result should be sent to MSOC;

proc freq data=diag_demo6 noprint ;
  by vaccine_group vacc_comb;
  tables dx*dx_wkday/missing out=msoc.dx_wkday;
    tables orig_dx*dx_wkday/missing out=msoc.orig_dx_wkday;
run;


*** Create crosswalk  - the file with list of variables needed for future data freeze
    for potential statistical(TreeScan) signal investigation;
*** Kept at DP;



 proc sql;
    create table dplocal.crswlk_for_freeze as
       select distinct vaccine_group, vacc_comb, patid, mlccs_incident, orig_dx, orig_dx_codetype, dx, dx_codetype, adate as dx_adate ,vacc_dt as exp_adate
         from diag_demo6;
 quit;


*** Create analytic result file for later use as an input for TreeScan software;

 proc sql;
	    create table MSOC.self_control_orig_dx as
   select distinct "&DPID" as DPID
              ,"&siteid" as siteid
              ,a.vaccine_group
			  ,b.vaccine_group_name
			  ,a.vacc_comb
			  ,a.orig_dx
              ,a.orig_dx_codetype
              ,a.dx
              ,a.dx_codetype
              ,a.days_from_exp_to_outcome
               ,count(*) as dose_cnt
   from diag_demo6 as a, general_input as b
    where a.vaccine_group=b.vaccine_group
    group by a.vaccine_group, vacc_comb, orig_dx, orig_dx_codetype, days_from_exp_to_outcome
       order by a.vaccine_group, vacc_comb, orig_dx, orig_dx_codetype, days_from_exp_to_outcome;
	  
 quit;


%ms_stoptimer(START=&MAINSTART.);
%create_signature;
